CREATE TABLE IF NOT EXISTS `CFG_ServerHistory`
(
    `Id`                      BIGINT(20)  NOT NULL,
    `MessageDeleteTimer`      BIGINT      NOT NULL DEFAULT 6,
    `NotificationChatId`      BIGINT      NOT NULL,
    `MaxVoiceStateHours`      BIGINT      NOT NULL DEFAULT 6,
    `XpPerMessage`            BIGINT      NOT NULL DEFAULT 1,
    `XpPerReaction`           BIGINT      NOT NULL DEFAULT 1,
    `MaxMessageXpPerHour`     BIGINT      NOT NULL DEFAULT 20,
    `XpPerOntimeHour`         BIGINT      NOT NULL DEFAULT 10,
    `XpPerEventParticipation` BIGINT      NOT NULL DEFAULT 10,
    `XpPerAchievement`        BIGINT      NOT NULL DEFAULT 10,
    `AFKCommandChannelId`     BIGINT      NOT NULL,
    `HelpVoiceChannelId`      BIGINT      NOT NULL,
    `TeamChannelId`           BIGINT      NOT NULL,
    `LoginMessageChannelId`   BIGINT      NOT NULL,
    `DefaultRoleId`           BIGINT      NULL,
    `FeatureFlags`            JSON        NULL     DEFAULT ('{}'),
    `ServerId`                BIGINT      NOT NULL,
    `Deleted`                 BOOL                 DEFAULT FALSE,
    `DateFrom`                DATETIME(6) NOT NULL,
    `DateTo`                  DATETIME(6) NOT NULL
);

DROP TRIGGER IF EXISTS `TR_CFG_ServerUpdate`;

CREATE TRIGGER `TR_CFG_ServerUpdate`
    AFTER UPDATE
    ON `CFG_Server`
    FOR EACH ROW
BEGIN
    INSERT INTO `CFG_ServerHistory` (`Id`,
                                     `MessageDeleteTimer`,
                                     `NotificationChatId`,
                                     `MaxVoiceStateHours`,
                                     `XpPerMessage`,
                                     `XpPerReaction`,
                                     `MaxMessageXpPerHour`,
                                     `XpPerOntimeHour`,
                                     `XpPerEventParticipation`,
                                     `XpPerAchievement`,
                                     `AFKCommandChannelId`,
                                     `HelpVoiceChannelId`,
                                     `TeamChannelId`,
                                     `LoginMessageChannelId`,
                                     `DefaultRoleId`,
                                     `FeatureFlags`,
                                     `ServerId`,
                                     `DateFrom`,
                                     `DateTo`)
    VALUES (OLD.Id,
            OLD.MessageDeleteTimer,
            OLD.NotificationChatId,
            OLD.MaxVoiceStateHours,
            OLD.XpPerMessage,
            OLD.XpPerReaction,
            OLD.MaxMessageXpPerHour,
            OLD.XpPerOntimeHour,
            OLD.XpPerEventParticipation,
            OLD.XpPerAchievement,
            OLD.AFKCommandChannelId,
            OLD.HelpVoiceChannelId,
            OLD.TeamChannelId,
            OLD.LoginMessageChannelId,
            OLD.DefaultRoleId,
            OLD.FeatureFlags,
            OLD.ServerId,
            OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;

DROP TRIGGER IF EXISTS `TR_CFG_ServerDelete`;

CREATE TRIGGER `TR_CFG_ServerDelete`
    AFTER DELETE
    ON `CFG_Server`
    FOR EACH ROW
BEGIN
    INSERT INTO `CFG_ServerHistory` (`Id`,
                                     `MessageDeleteTimer`,
                                     `NotificationChatId`,
                                     `MaxVoiceStateHours`,
                                     `XpPerMessage`,
                                     `XpPerReaction`,
                                     `MaxMessageXpPerHour`,
                                     `XpPerOntimeHour`,
                                     `XpPerEventParticipation`,
                                     `XpPerAchievement`,
                                     `AFKCommandChannelId`,
                                     `HelpVoiceChannelId`,
                                     `TeamChannelId`,
                                     `LoginMessageChannelId`,
                                     `DefaultRoleId`,
                                     `ServerId`,
                                     `FeatureFlags`,
                                     `Deleted`,
                                     `DateFrom`,
                                     `DateTo`)
    VALUES (OLD.Id,
            OLD.MessageDeleteTimer,
            OLD.NotificationChatId,
            OLD.MaxVoiceStateHours,
            OLD.XpPerMessage,
            OLD.XpPerReaction,
            OLD.MaxMessageXpPerHour,
            OLD.XpPerOntimeHour,
            OLD.XpPerEventParticipation,
            OLD.XpPerAchievement,
            OLD.AFKCommandChannelId,
            OLD.HelpVoiceChannelId,
            OLD.TeamChannelId,
            OLD.LoginMessageChannelId,
            OLD.DefaultRoleId,
            OLD.FeatureFlags,
            OLD.ServerId,
            TRUE,
            OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;