from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class ShortRoleNameMigration(MigrationABC):
    name = "1.1.7_ShortRoleNameMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `ShortRoleNames` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `ShortName` VARCHAR(255) NOT NULL,
                        `DiscordRoleId` BIGINT NOT NULL,
                        `Position` ENUM('Before', 'After') NOT NULL,
                        `ServerId` BIGINT,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`),
                        FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
                    );
                """
            )
        )

        self._exec(__file__, "short_rule_names.sql")

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ShortRoleNames`;")
        self._cursor.execute("DROP TABLE `ShortRoleNamesHistory`;")
