from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.short_role_name import ShortRoleName
from bot_data.model.short_role_name_position_enum import ShortRoleNamePositionEnum
from bot_data.model.user import User
from bot_graphql.abc.filter_abc import FilterABC


class ShortRoleNameFilter(FilterABC):
    def __init__(self, bot: DiscordBotServiceABC):
        FilterABC.__init__(self)
        self._bot = bot

        self._id = None
        self._short_name = None
        self._role_id = None
        self._role_name = None
        self._position = None
        self._server = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "shortName" in values:
            self._short_name = values["shortName"]

        if "roleId" in values:
            self._role_id = int(values["roleId"])

        if "roleName" in values:
            self._role_name = values["roleName"]

        if "position" in values:
            self._position = ShortRoleNamePositionEnum(values["position"])

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            self._server: ServerFilter = self._services.get_service(ServerFilter)
            self._server.from_dict(values["server"])

    def filter(self, query: List[User]) -> List[User]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._short_name is not None:
            query = query.where(lambda x: x.short_name == self._short_name or self._short_name in x.short_name)

        if self._role_id is not None:
            query = query.where(lambda x: x.role_id == self._role_id)

        if self._role_name is not None and self._role_id is not None:

            def get_role_name(x: ShortRoleName):
                guild = self._bot.get_guild(x.server.discord_id)
                name = guild.get_role(x.role_id).name
                return name == self._role_name or self._role_name in name

            query = query.where(get_role_name)

        if self._position is not None:
            query = query.where(lambda x: x.position == self._position or self._position in x.position)

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.id)
            query = query.where(lambda x: x.server.id in servers)

        return query
