from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_data.service.seeder_service import SeederService
from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.abc.query_abc import QueryABC
from bot_graphql.filter.achievement_filter import AchievementFilter
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.auto_role_rule_filter import AutoRoleRuleFilter
from bot_graphql.filter.client_filter import ClientFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.server_filter import ServerFilter
from bot_graphql.filter.short_role_name_filter import ShortRoleNameFilter
from bot_graphql.filter.user_filter import UserFilter
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import UserJoinedVoiceChannelFilter
from bot_graphql.graphql_service import GraphQLService
from bot_graphql.mutation import Mutation
from bot_graphql.mutations.achievement_mutation import AchievementMutation
from bot_graphql.mutations.auto_role_mutation import AutoRoleMutation
from bot_graphql.mutations.auto_role_rule_mutation import AutoRoleRuleMutation
from bot_graphql.mutations.level_mutation import LevelMutation
from bot_graphql.mutations.server_config_mutation import ServerConfigMutation
from bot_graphql.mutations.technician_config_mutation import TechnicianConfigMutation
from bot_graphql.mutations.user_joined_game_server_mutation import UserJoinedGameServerMutation
from bot_graphql.mutations.user_mutation import UserMutation
from bot_graphql.queries.achievement_attribute_query import AchievementAttributeQuery
from bot_graphql.queries.achievement_history_query import AchievementHistoryQuery
from bot_graphql.queries.achievement_query import AchievementQuery
from bot_graphql.queries.auto_role_history_query import AutoRoleHistoryQuery
from bot_graphql.queries.auto_role_query import AutoRoleQuery
from bot_graphql.queries.auto_role_rule_history_query import AutoRoleRuleHistoryQuery
from bot_graphql.queries.auto_role_rule_query import AutoRoleRuleQuery
from bot_graphql.queries.client_history_query import ClientHistoryQuery
from bot_graphql.queries.client_query import ClientQuery
from bot_graphql.queries.discord.channel_query import ChannelQuery
from bot_graphql.queries.discord.discord_query import DiscordQuery
from bot_graphql.queries.discord.discord_user_query import DiscordUserQuery
from bot_graphql.queries.discord.emoji_query import EmojiQuery
from bot_graphql.queries.discord.guild_query import GuildQuery
from bot_graphql.queries.discord.role_query import RoleQuery
from bot_graphql.queries.game_server_query import GameServerQuery
from bot_graphql.queries.known_user_history_query import KnownUserHistoryQuery
from bot_graphql.queries.known_user_query import KnownUserQuery
from bot_graphql.queries.level_history_query import LevelHistoryQuery
from bot_graphql.queries.level_query import LevelQuery
from bot_graphql.queries.server_config_query import ServerConfigQuery
from bot_graphql.queries.server_history_query import ServerHistoryQuery
from bot_graphql.queries.server_query import ServerQuery
from bot_graphql.queries.short_role_name_history_query import ShortRoleNameHistoryQuery
from bot_graphql.queries.short_role_name_query import ShortRoleNameQuery
from bot_graphql.queries.technician_config_history_query import TechnicianConfigHistoryQuery
from bot_graphql.queries.technician_config_query import TechnicianConfigQuery
from bot_graphql.queries.technician_id_config_history_query import TechnicianIdConfigHistoryQuery
from bot_graphql.queries.technician_ping_url_config_history_query import TechnicianPingUrlConfigHistoryQuery
from bot_graphql.queries.user_history_query import UserHistoryQuery
from bot_graphql.queries.user_joined_game_server_history_query import UserJoinedGameServerHistoryQuery
from bot_graphql.queries.user_joined_game_server_query import UserJoinedGameServerQuery
from bot_graphql.queries.user_joined_server_history_query import UserJoinedServerHistoryQuery
from bot_graphql.queries.user_joined_server_query import UserJoinedServerQuery
from bot_graphql.queries.user_joined_voice_channel_history_query import UserJoinedVoiceChannelHistoryQuery
from bot_graphql.queries.user_joined_voice_channel_query import UserJoinedVoiceChannelQuery
from bot_graphql.queries.user_query import UserQuery
from bot_graphql.query import Query
from bot_graphql.schema import Schema


class GraphQLModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.data_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_singleton(Schema)
        services.add_singleton(GraphQLService)
        services.add_singleton(Query)
        services.add_singleton(Mutation)

        # queries
        services.add_transient(QueryABC, AchievementAttributeQuery)
        services.add_transient(QueryABC, AchievementQuery)
        services.add_transient(QueryABC, AchievementHistoryQuery)
        services.add_transient(QueryABC, AutoRoleHistoryQuery)
        services.add_transient(QueryABC, AutoRoleQuery)
        services.add_transient(QueryABC, AutoRoleRuleHistoryQuery)
        services.add_transient(QueryABC, AutoRoleRuleQuery)
        services.add_transient(QueryABC, ClientHistoryQuery)
        services.add_transient(QueryABC, ClientQuery)
        services.add_transient(QueryABC, KnownUserHistoryQuery)
        services.add_transient(QueryABC, KnownUserQuery)
        services.add_transient(QueryABC, LevelHistoryQuery)
        services.add_transient(QueryABC, LevelQuery)
        services.add_transient(QueryABC, ServerConfigQuery)
        services.add_transient(QueryABC, ServerHistoryQuery)
        services.add_transient(QueryABC, ServerQuery)
        services.add_transient(QueryABC, TechnicianConfigQuery)
        services.add_transient(QueryABC, TechnicianConfigHistoryQuery)
        services.add_transient(QueryABC, TechnicianPingUrlConfigHistoryQuery)
        services.add_transient(QueryABC, TechnicianIdConfigHistoryQuery)
        services.add_transient(QueryABC, GameServerQuery)
        services.add_transient(QueryABC, UserHistoryQuery)
        services.add_transient(QueryABC, UserQuery)
        services.add_transient(QueryABC, UserJoinedServerHistoryQuery)
        services.add_transient(QueryABC, UserJoinedServerQuery)
        services.add_transient(QueryABC, UserJoinedVoiceChannelHistoryQuery)
        services.add_transient(QueryABC, UserJoinedVoiceChannelQuery)
        services.add_transient(QueryABC, UserJoinedGameServerHistoryQuery)
        services.add_transient(QueryABC, UserJoinedGameServerQuery)
        services.add_transient(QueryABC, ShortRoleNameHistoryQuery)
        services.add_transient(QueryABC, ShortRoleNameQuery)

        services.add_transient(QueryABC, DiscordQuery)
        services.add_transient(QueryABC, GuildQuery)
        services.add_transient(QueryABC, ChannelQuery)
        services.add_transient(QueryABC, RoleQuery)
        services.add_transient(QueryABC, EmojiQuery)
        services.add_transient(QueryABC, DiscordUserQuery)

        # filters
        services.add_transient(FilterABC, AutoRoleFilter)
        services.add_transient(FilterABC, AutoRoleRuleFilter)
        services.add_transient(FilterABC, ClientFilter)
        services.add_transient(FilterABC, LevelFilter)
        services.add_transient(FilterABC, ServerFilter)
        services.add_transient(FilterABC, UserFilter)
        services.add_transient(FilterABC, AchievementFilter)
        services.add_transient(FilterABC, UserJoinedServerFilter)
        services.add_transient(FilterABC, UserJoinedVoiceChannelFilter)
        services.add_transient(FilterABC, UserJoinedGameServerFilter)
        services.add_transient(FilterABC, ShortRoleNameFilter)

        # mutations
        services.add_transient(QueryABC, AutoRoleMutation)
        services.add_transient(QueryABC, AutoRoleRuleMutation)
        services.add_transient(QueryABC, LevelMutation)
        services.add_transient(QueryABC, UserMutation)
        services.add_transient(QueryABC, AchievementMutation)
        services.add_transient(QueryABC, UserJoinedGameServerMutation)
        services.add_transient(QueryABC, TechnicianConfigMutation)
        services.add_transient(QueryABC, ServerConfigMutation)

        services.add_transient(SeederService)
