import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.achievements.achievement_service import AchievementService


class AchievementOnVoiceStateUpdateEvent(OnVoiceStateUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        achievements: AchievementService,
        db: DatabaseContextABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
    ):
        OnVoiceStateUpdateABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._achievements = achievements
        self._db = db
        self._servers = servers
        self._users = users

    async def on_voice_state_update(
        self, member: discord.member.Member, before: discord.member.VoiceState, after: discord.member.VoiceState
    ):
        if member.guild is None:
            return

        if member.bot:
            return

        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(
            server_config.feature_flags, FeatureFlagsEnum.achievements_module
        ):
            return

        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)

        await self._achievements.validate_achievements_for_user(user)
