import discord
from cpl_core.configuration import ConfigurationABC
from cpl_discord.events import OnMemberJoinABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_core.logging.message_logger import MessageLogger
from bot_data.model.server_config import ServerConfig
from modules.level.service.level_service import LevelService


class LevelOnMemberJoinEvent(OnMemberJoinABC):
    def __init__(self, config: ConfigurationABC, logger: MessageLogger, level: LevelService):
        OnMemberJoinABC.__init__(self)
        self._config = config
        self._logger = logger
        self._level = level

    @EventChecks.check_is_ready()
    async def on_member_join(self, member: discord.Member):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(server_config.feature_flags, FeatureFlagsEnum.level_module):
            return

        await self._level.check_level(member)
