from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events.on_raw_reaction_add_abc import OnRawReactionAddABC
from cpl_discord.service import DiscordBotServiceABC
from discord import RawReactionActionEvent

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_data.model.server_config import ServerConfig
from modules.level.service.level_service import LevelService


class LevelOnRawReactionAddEvent(OnRawReactionAddABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        level: LevelService,
    ):
        OnRawReactionAddABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._level = level

    @EventChecks.check_is_ready()
    async def on_raw_reaction_add(self, payload: RawReactionActionEvent):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{payload.guild_id}")
        if not FeatureFlagsSettings.get_flag_from_dict(server_config.feature_flags, FeatureFlagsEnum.level_module):
            return

        try:
            self._logger.trace(__name__, f"Handle reaction {payload} for level")

            guild = self._bot.get_guild(payload.guild_id)
            member = guild.get_member(payload.user_id)

            await self._level.check_level(member)
        except Exception as e:
            self._logger.error(__name__, f"Level check by message failed", e)
